#Script for reproduction of primary results as discussed in "Integration of Mechanistic Immunological Knowledge into a Machine Learning Pipeline Increases Clinical Predictive Power"

#Installation of package from source and other necessary packages
install.packages("iEN_0.99.0.tar.gz", repos = NULL, type="source")
install.packages("caret", repos='http://cran.us.r-project.org')
install.packages("parallel", repos='http://cran.us.r-project.org')

library("iEN")
library("caret") #required for fold creation
library("parallel") #required for detection of cores

load("ien_data.rda")

#repeated K-fold analysis for pregnancy dataset
X <- preg.X
Y <- preg.Y
priors <- preg.priors
alphaGrid <- seq(0,1, length.out=11) 
phiGrid <- c(0, exp(seq(log(1),log(100), length.out=10))) 
nlambda <- 10
lambdas <- NULL #lambda sequence can be given to the method but was not for any of the analyses
eval <- "RSS" 
family <- "gaussian"
intercept <- TRUE
standardize <- TRUE
center <- TRUE

#NOTE: to decrease runtime reduce search grid values (alphaGrid, phiGrid, nlambda), this will reduce
#the time between runs but will also produce different model performance estimates

#Cross-Validation set up
K <- 10
repeats <- 10 #set to 100 in the paper
ncores <- detectCores()-1 #for optimal parallelization set to length(alphaGrid) * length(phiGrid)

preg.en.pear.pvs <- vector()
preg.ien.pear.pvs <- vector()

val.en.pear.pvs <- vector()
val.ien.pear.pvs <- vector()

set.seed(1994)
for(i in seq(repeats)){

	folds <- createFolds(unique(preg.foldid),k=K)

	#not directly taking folds from createFolds to maintain 
	#separation for patients giving multiple samples
	foldid <- vector()
	for(k in seq(length(folds))){
		foldid[which(preg.foldid %in% folds[[k]])] <- k
	}

	#EN phiGrid of zero sets optimization surface equal to EN
	en.ans=cv_iEN(X, Y, foldid, alphaGrid, phiGrid=c(0), nlambda, lambdas,  priors, ncores, eval, family, intercept, standardize)
	ien.ans=cv_iEN(X, Y, foldid, alphaGrid, phiGrid, nlambda, lambdas,  priors, ncores, eval, family, intercept, standardize)

	preg.en.pear.pvs[i] <- cor.test(Y, en.ans@cv.preds, method="pearson")$p.value
	preg.ien.pear.pvs[i] <- cor.test(Y, ien.ans@cv.preds, method="pearson")$p.value

	#predict on validation data
	en.val.preds <- predict(en.ans, v.preg.X, priors, 0)
	ien.val.preds <- predict(ien.ans, v.preg.X, priors)

	val.en.pear.pvs[i] <- cor.test(en.val.preds, v.preg.Y,method="pearson")$p.value
	val.ien.pear.pvs[i] <- cor.test(ien.val.preds, v.preg.Y,method="pearson")$p.value
	

	if(i == 1){
		cat(sprintf('-log10 Pearson P-value:\n\nTraining Performance \t\t\t Validation Performance\niEN: %f, EN: %f \t\tiEN: %f, EN: %f\n', -log10(preg.ien.pear.pvs[i]), -log10(preg.en.pear.pvs[i]),-log10(val.ien.pear.pvs[i]), -log10(val.en.pear.pvs[i])))
	}else{
		cat(sprintf('iEN: %f, EN: %f \t\tiEN: %f, EN: %f\n', -log10(preg.ien.pear.pvs[i]), -log10(preg.en.pear.pvs[i]),-log10(val.ien.pear.pvs[i]), -log10(val.en.pear.pvs[i])))
	}
	
}

closeAllConnections()

preg.pvs <- data.frame(cbind(preg.ien.pear.pvs, preg.en.pear.pvs))
colnames(preg.pvs) <- c("iEN","EN")

val.pvs <- data.frame(cbind(val.ien.pear.pvs, val.en.pear.pvs))
colnames(val.pvs) <- c("iEN","EN")

boxplot(-log10(preg.pvs))
boxplot(-log10(val.pvs))



###################################################
#repeated K-fold analysis for periodontitis dataset
X <- perio.X
Y <- perio.Y
priors <- perio.priors
alphaGrid <- seq(0,1, length.out=11)
phiGrid <- c(0, exp(seq(log(1),log(100), length.out=10)))
nlambda <- 10
lambdas <- NULL
eval <- "ROCAUC" 
family <- "binomial"
intercept <- FALSE
standardize <- TRUE
center <- TRUE


K <- 10
repeats <- 10
ncores <- detectCores()-1 

perio.en.wilc.pvs <- vector()
perio.ien.wilc.pvs <- vector()

set.seed(1994)
for(i in seq(repeats)){
	
	foldid <- createFolds(seq(nrow(perio.X)),k=K,list=FALSE)

	en.ans=cv_iEN(X, Y, foldid, alphaGrid, phiGrid=c(0), nlambda, lambdas,  priors, ncores, eval, family, intercept, standardize, center)
	ien.ans=cv_iEN(X, Y, foldid, alphaGrid, phiGrid, nlambda, lambdas,  priors, ncores, eval, family, intercept, standardize, center)

	perio.en.wilc.pvs[i] <- wilcox.test(en.ans@cv.preds~Y)$p.value
	perio.ien.wilc.pvs[i] <- wilcox.test(ien.ans@cv.preds~Y)$p.value


	if(i == 1){
		cat(sprintf('-log10 Wilcoxon P-value:\n\niEN: %f, EN: %f\n', -log10(perio.ien.wilc.pvs[i]), -log10(perio.en.wilc.pvs[i])))
	}else{
		cat(sprintf('iEN: %f, EN: %f\n', -log10(perio.ien.wilc.pvs[i]), -log10(perio.en.wilc.pvs[i])))
	}
	
}

closeAllConnections()

perio.pvs <- cbind(perio.ien.wilc.pvs, perio.en.wilc.pvs)
colnames(perio.pvs) <- c("iEN","EN")

boxplot(-log10(perio.pvs))
